/*""FILE COMMENT""**************************************************************
*
*       System Name : SH7670 USB Sample Program
*       File Name   : h_controlrw.c
*       Contents    : Host Control Read/Write
*       Version     : 2.01.00
*       Model       : M3A-HS71
*       CPU         : SH7670
*       Compiler    : SHC9.1.1.0
*       OS          : none
*
*       note        :
*                     <ӎ>
*                     {TvvOׂ͂ĎQlłA
*                     ̓ۏ؂̂ł͂܂B
*                     {TvvO͂ql̃\tgEGAJ
*                     ZpQlƂĂp
*
*       The information described here may contain technical inaccuracies or
*       typographical errors. Renesas Technology Corporation and Renesas Solutions
*       assume no responsibility for any damage, liability, or other loss rising 
*       from these inaccuracies or errors.
*
*       Copyright (C) 2008 Renesas Technology Corp. All Rights Reserved
*       AND Renesas Solutions Corp. All Rights Reserved
*
*       history   : 2008.01.09 ver.1.00.00 
*                 : 2008.01.17 ver.2.00.00 
*                 : 2008.09.02 ver.2.01.00 
*""FILE COMMENT END""*********************************************************/

/******* include file ********************************************************/
#include "..\..\inc\usb.h"

/*""FUNC COMMENT""************************************************************ 
* ID         : 
* Outline    : Control Read/Write/No-data
*----------------------------------------------------------------------------- 
* Include    : 
*----------------------------------------------------------------------------- 
* Declaration: USB_U16 H_CtrlTransStart( USB_U16 Req, USB_U16 Val, USB_U16 Indx,
*            :                           USB_U16 Len, USB_U8 *Buf)
*----------------------------------------------------------------------------- 
* Function   : 
*            : 
*----------------------------------------------------------------------------- 
* Argument   : USB_U16 Req			; bmRequestType & bRequest
*            : USB_U16 Val			; wValue
*            : USB_U16 Indx			; wIndex
*            : USB_U16 Len			; wLength
*            : USB_U8  *Buf			; Data buffer
*----------------------------------------------------------------------------- 
* ReturnValue: USB_SUCCESS	:	SUCCESS
*            : USB_ERROR	:	ERROR
*----------------------------------------------------------------------------- 
* Notice     : 
*            : 
*""FUNC COMMENT END""*********************************************************/
USB_U16 H_CtrlTransStart(USB_U16 Req, USB_U16 Val, 
							USB_U16 Indx, USB_U16 Len, USB_U8 *Buf)
{
	if( UsbDeviceSpeed == LOW_SPEED ){
		USB.SOFCFG.BIT.TRNENSEL = 1;
	}
	else{
		USB.SOFCFG.BIT.TRNENSEL = 0;
	}

	USB.DCPMAXP.WORD = ( USB_U16 )( ( USB_U16 )( UsbAddress << 12 ) + UsbMaxPacketSize); 

	if( PipeFlag[PIPE0] == PIPE_IDLE ) {
		PipeFlag[PIPE0] = PIPE_WAIT;
		PipeIgnore[PIPE0]	= 0;					/* Ignore count clear */
		UsbCmdStage = (STAGE_SETUP | CMD_IDLE);
		if( Len == 0 ) {
				UsbCmdStage |= MODE_NO_DATA;		/* No-data Control */
		} 
		else {
			if( (Req & 0x0080) != 0 ) {
				UsbCmdStage |= MODE_READ;			/* Control Read */
			} 
			else {
				UsbCmdStage |= MODE_WRITE;			/* Control Write */
			}
		}

		UsbSavReq	= Req;							/* save request */
		UsbSavVal	= Val;
		UsbSavIndx	= Indx;
		UsbSavLen	= Len;
	} 
	else {
		if( (UsbSavReq  != Req)  || (UsbSavVal != Val)
		 || (UsbSavIndx != Indx) || (UsbSavLen != Len) ) {
			return (USB_U16)USB_ERROR;
		}
	}

	switch( (UsbCmdStage & (STAGE_FIELD | CMD_FIELD)) )
	{
	/* --------------- SETUP STAGE --------------- */
	case (STAGE_SETUP | CMD_IDLE):
		H_SetupStage( Req, Val, Indx, Len);
		break;

	case (STAGE_SETUP | CMD_DOING):
		/* do nothing */
		break;

	case (STAGE_SETUP | CMD_DONE):					/* goto next stage */
		PipeIgnore[PIPE0]	= 0;					/* Ignore count clear */
		switch( (UsbCmdStage & (MODE_FIELD)) ){
		case MODE_WRITE:
			CHG_STAGE_FIELD(UsbCmdStage, STAGE_DATA);
			break;
		case MODE_READ:
			CHG_STAGE_FIELD(UsbCmdStage, STAGE_DATA);
			break;
		case MODE_NO_DATA:
			CHG_STAGE_FIELD(UsbCmdStage, STAGE_STATUS); 
			break;
		}
		CHG_CMD_FIELD(UsbCmdStage, CMD_IDLE);
		break;

	case (STAGE_SETUP | CMD_NORES):
		if(PipeIgnore[PIPE0] == PIPEERR){
			PipeFlag[PIPE0] = PIPE_NORES;			/* exit NORES */
		} 
		else {
			PipeIgnore[PIPE0]++;					/* Ignore count */
			CHG_CMD_FIELD(UsbCmdStage, CMD_IDLE);	/* retry */
		}
		break;

	/* --------------- DATA STAGE --------------- */
	case (STAGE_DATA | CMD_IDLE):
		switch( (UsbCmdStage & (MODE_FIELD)) ){
		case MODE_WRITE:
			H_CtrlWriteStart((USB_U32)Len, Buf); 
			break;
		case MODE_READ:
			H_CtrlReadStart((USB_U32)Len, Buf);
			break;
		}
		break;

	case (STAGE_DATA | CMD_DOING):
		/* do nothing */
		break;

	case (STAGE_DATA | CMD_DONE):					/* goto next stage */
		PipeIgnore[PIPE0]	= 0;					/* Ignore count clear */
		CHG_STAGE_FIELD(UsbCmdStage, STAGE_STATUS);
		CHG_CMD_FIELD(UsbCmdStage, CMD_IDLE);
		break;

	case (STAGE_DATA | CMD_NORES):
		if(PipeIgnore[PIPE0] == PIPEERR){
			PipeFlag[PIPE0] = PIPE_NORES;			/* exit NORES */
		} 
		else {
			PipeIgnore[PIPE0]++;					/* Ignore count */
			CHG_CMD_FIELD(UsbCmdStage, CMD_DOING);	/* retry */
			usb_clear_pid_stall(PIPE0);
			usb_set_pid_buf(PIPE0);
		}
		break;

	case (STAGE_DATA | CMD_STALL):
		PipeFlag[PIPE0] = PIPE_STALL;				/* exit STALL */
		break;

	/* --------------- STATUS STAGE --------------- */
	case (STAGE_STATUS | CMD_IDLE):
		H_StatusStage();
		break;

	case (STAGE_STATUS | CMD_DOING):
		/* do nothing */
		break;

	case (STAGE_STATUS | CMD_DONE):					/* end of Control transfer */
		usb_set_pid_nak(PIPE0);
		PipeFlag[PIPE0] = PIPE_DONE;				/* exit DONE */
		break;

	case (STAGE_STATUS | CMD_NORES):
		if(PipeIgnore[PIPE0] == PIPEERR){
			PipeFlag[PIPE0] = PIPE_NORES;			/* exit NORES */
		} 
		else {
			PipeIgnore[PIPE0]++;					/* Ignore count */
			CHG_CMD_FIELD(UsbCmdStage, CMD_DOING);	/* retry */
			usb_clear_pid_stall(PIPE0);
			usb_set_pid_buf(PIPE0);
		}
		break;

	case (STAGE_STATUS | CMD_STALL):
		PipeFlag[PIPE0] = PIPE_STALL;				/* exit STALL */
		break;
	}

	if( PipeFlag[ PIPE0 ] != PIPE_WAIT ){
		USB.SOFCFG.BIT.TRNENSEL = 0;
	}
	return USB_SUCCESS;
}

/*""FUNC COMMENT""************************************************************ 
* ID         : 
* Outline    : Setup stage
*----------------------------------------------------------------------------- 
* Include    : 
*----------------------------------------------------------------------------- 
* Declaration: void H_SetupStage(USB_U16 Req, USB_U16 Val, USB_U16 Indx, USB_U16 Len)
*----------------------------------------------------------------------------- 
* Function   : 
*            : 
*----------------------------------------------------------------------------- 
* Argument   : USB_U16 Req			; bmRequestType & bRequest
*            : USB_U16 Val			; wValue
*            : USB_U16 Indx			; wIndex
*            : USB_U16 Len			; wLength
*----------------------------------------------------------------------------- 
* ReturnValue: void
*----------------------------------------------------------------------------- 
* Notice     : 
*            : 
*""FUNC COMMENT END""*********************************************************/
void H_SetupStage( USB_U16 Req, USB_U16 Val, USB_U16 Indx, USB_U16 Len)
{
	CHG_CMD_FIELD(UsbCmdStage, CMD_DOING);

	USB.INTSTS1.WORD = (USB_U16)~(BITSACK | BITSIGN);	/* Status Clear */
	USB.USBREQ.WORD = Req;
	USB.USBVAL.WORD = Val;
	USB.USBINDX.WORD = Indx;
	USB.USBLENG.WORD = Len;
	USB.DCPCTR.WORD = BITSUREQ;						/* PID=NAK & Send Setup */
}

/*""FUNC COMMENT""************************************************************ 
* ID         : 
* Outline    : Status stage
*----------------------------------------------------------------------------- 
* Include    : 
*----------------------------------------------------------------------------- 
* Declaration: void H_StatusStage(void)
*----------------------------------------------------------------------------- 
* Function   : 
*            : 
*----------------------------------------------------------------------------- 
* Argument   : void
*----------------------------------------------------------------------------- 
* ReturnValue: void
*----------------------------------------------------------------------------- 
* Notice     : 
*            : 
*""FUNC COMMENT END""*********************************************************/
void H_StatusStage( void )
{
	USB_U8	Buf1[16];

	switch( (UsbCmdStage & (MODE_FIELD)) )
	{
		case MODE_READ:
			H_CtrlWriteStart((USB_U32)0, (USB_U8*)&Buf1); 
			break;
		case MODE_WRITE:
			H_CtrlReadStart((USB_U32)0, (USB_U8*)&Buf1); 
			break;
		case MODE_NO_DATA:
			H_CtrlReadStart((USB_U32)0, (USB_U8*)&Buf1); 
			break;
		default:
			break;
	}
}

/*""FUNC COMMENT""************************************************************ 
* ID         : 
* Outline    : Control write start
*----------------------------------------------------------------------------- 
* Include    : 
*----------------------------------------------------------------------------- 
* Declaration: USB_U16 H_CtrlWriteStart(USB_U32 Bsize, USB_U8 *Table)
*----------------------------------------------------------------------------- 
* Function   : 
*            : 
*----------------------------------------------------------------------------- 
* Argument   : USB_U32 Bsize		; Data Size
*            : USB_U8  *Table		; Data Table Address
*----------------------------------------------------------------------------- 
* ReturnValue: WRITESHRT	;	End of data write
*            : WRITEEND		;	End of data write (not null)
*            : WRITING		;	Continue of data write
*            : FIFOERROR	;	FIFO access error
*----------------------------------------------------------------------------- 
* Notice     : 
*            : 
*""FUNC COMMENT END""*********************************************************/
USB_U16 H_CtrlWriteStart( USB_U32 Bsize, USB_U8 *Table)
{
	USB_U16	EndFlag_K, mbw;

	CHG_CMD_FIELD(UsbCmdStage, CMD_DOING);

	usb_set_pid_nak(PIPE0);									/* Set NAK */
	USBDataCount[PIPE0]   = Bsize;							/* Transfer size set */
	USBDataPointer[PIPE0] = Table;							/* Transfer address set */

	USB.DCPCTR.WORD = BITSQSET;					/* SQSET=1, PID=NAK */
	USB.DCPCFG.BIT.DIR = 1;

	mbw = usb_get_mbw( USBDataCount[PIPE0], (USB_U32)USBDataPointer[PIPE0]);
	usb_set_curpipe(PIPE0,CUSE,BITISEL,mbw);
	USB.CFIFOCTR.WORD = BITBCLR;					/* Buffer Clear */

	usb_clear_pid_stall(PIPE0);
	EndFlag_K	= usb_write_buffer_c(PIPE0);
	/* Host Control sequence */
	switch( EndFlag_K ){
	case	WRITESHRT:								/* End of data write */
		CHG_STAGE_FIELD(UsbCmdStage, STAGE_STATUS);
		usb_enable_bemp_int(PIPE0);					/* Enable Empty Interrupt */
		usb_enable_nrdy_int(PIPE0);					/* Error (NORES or STALL) */
		break;
	case	WRITEEND:								/* End of data write (not null) */
	case	WRITING:								/* Continue of data write */
		usb_enable_bemp_int(PIPE0);					/* Enable Empty Interrupt */
		usb_enable_nrdy_int(PIPE0);					/* Error (NORES or STALL) */
		break;
	case	FIFOERROR:								/* FIFO access error */
		break;
	default:
		break;
	}
	usb_set_pid_buf(PIPE0);									/* Set BUF */
	return	(EndFlag_K);							/* End or Err or Continue */
}

/*""FUNC COMMENT""************************************************************ 
* ID         : 
* Outline    : Control read start
*----------------------------------------------------------------------------- 
* Include    : 
*----------------------------------------------------------------------------- 
* Declaration: void H_CtrlReadStart(USB_U32 Bsize, USB_U8 *Table)
*----------------------------------------------------------------------------- 
* Function   : 
*            : 
*----------------------------------------------------------------------------- 
* Argument   : USB_U32 Bsize		; Data Size
*            : USB_U8  *Table		; Data Table Address
*----------------------------------------------------------------------------- 
* ReturnValue: void
*----------------------------------------------------------------------------- 
* Notice     : 
*            : 
*""FUNC COMMENT END""*********************************************************/
void H_CtrlReadStart( USB_U32 Bsize, USB_U8 *Table)
{
	USB_U16 mbw;

	CHG_CMD_FIELD(UsbCmdStage, CMD_DOING);

	usb_set_pid_nak(PIPE0);								/* Set NAK */
	USBDataCount[PIPE0]    = Bsize;						/* Transfer size set */
	USBDataPointer[PIPE0]  = Table;						/* Transfer address set */
	USBRcvDataCount[PIPE0] = 0;

	USB.DCPCTR.WORD     = BITSQSET;					/* SQSET=1, PID=NAK */
	USB.DCPCFG.BIT.DIR  = 0;

	mbw = usb_get_mbw( USBDataCount[PIPE0], (USB_U32)USBDataPointer[PIPE0]);
	usb_set_curpipe(PIPE0,CUSE,USB_NO,mbw);
	USB.CFIFOCTR.WORD = BITBCLR;						/* Buffer Clear */

	usb_enable_brdy_int(PIPE0);							/* Ok */
	usb_enable_nrdy_int(PIPE0);							/* Error (NORES or STALL) */
	usb_clear_pid_stall(PIPE0);
	usb_set_pid_buf(PIPE0);								/* Set BUF */
}

/*===========================================================================*/
/* End of File */
