/*""FILE COMMENT""**************************************************************
*
*       System Name : SH7670 USB Sample Program
*       File Name   : h_usbdrv_api.c
*       Contents    : Host USB Class Driver
*       Version     : 2.01.00
*       Model       : M3A-HS71
*       CPU         : SH7670
*       Compiler    : SHC9.1.1.0
*       OS          : none
*
*       note        :
*                     <ӎ>
*                     {TvvOׂ͂ĎQlłA
*                     ̓ۏ؂̂ł͂܂B
*                     {TvvO͂ql̃\tgEGAJ
*                     ZpQlƂĂp
*
*       The information described here may contain technical inaccuracies or
*       typographical errors. Renesas Technology Corporation and Renesas Solutions
*       assume no responsibility for any damage, liability, or other loss rising 
*       from these inaccuracies or errors.
*
*       Copyright (C) 2008 Renesas Technology Corp. All Rights Reserved
*       AND Renesas Solutions Corp. All Rights Reserved
*
*       history   : 2008.01.09 ver.1.00.00 
*                 : 2008.01.17 ver.2.00.00 
*                 : 2008.09.02 ver.2.01.00 
*""FILE COMMENT END""*********************************************************/

/******* include file ********************************************************/
#include "..\..\inc\usb.h"

/*""FUNC COMMENT""************************************************************ 
* ID         : 
* Outline    : Host mode initialize
*----------------------------------------------------------------------------- 
* Include    : 
*----------------------------------------------------------------------------- 
* Declaration: void H_ModeInit(void)
*----------------------------------------------------------------------------- 
* Function   : 
*            : 
*----------------------------------------------------------------------------- 
* Argument   : void
*----------------------------------------------------------------------------- 
* ReturnValue: void
*----------------------------------------------------------------------------- 
* Notice     : 
*            : 
*""FUNC COMMENT END""*********************************************************/
void H_ModeInit(void)
{
	USB_U16	conect;

	usb_set_int_level(USBLVL);			/* Set interrupt Level	*/

	usb_reset_module();					/* Reset USB module 	*/

	/* Interrupt Flag for USB Driver	*/
	UsbBchgFlg = USB_NO;
	UsbDtchFlg = USB_NO;
	UsbAtchFlg = USB_NO;

	UsbDeviceState = DVST_DETACHED;

	UsbMaxPacketSize = 64;				/* 64byte 		*/

	H_InitModule();						/* Initialize USB module to HOST Mode	*/

	conect = H_CheckAttach();
	if( conect == ATTACH ) {
		UsbAtchFlg = USB_YES;
	} 
	else {
		UsbDtchFlg = USB_YES;
	}
}

/*""FUNC COMMENT""************************************************************ 
* ID         : 
* Outline    : Get Device State
*----------------------------------------------------------------------------- 
* Include    : 
*----------------------------------------------------------------------------- 
* Declaration: USB_U16 H_GetUsbDeviceState(void)
*----------------------------------------------------------------------------- 
* Function   : 
*            : 
*----------------------------------------------------------------------------- 
* Argument   : void
*----------------------------------------------------------------------------- 
* ReturnValue: UsbDeviceState	: devicestate
*----------------------------------------------------------------------------- 
* Notice     : 
*            : 
*""FUNC COMMENT END""*********************************************************/
USB_U16 H_GetUsbDeviceState( void )
{
	USB_U16		devsts;

	if( UsbDtchFlg == USB_YES ) {
		UsbDtchFlg = USB_NO;
		H_UsbDetach();
	}

	H_UsbStateManager();

	devsts = UsbDeviceState;

	return devsts;
}

/*""FUNC COMMENT""************************************************************ 
* ID         : 
* Outline    : User data receive
*----------------------------------------------------------------------------- 
* Include    : 
*----------------------------------------------------------------------------- 
* Declaration: USB_S16 H_DataIn(USB_U16 Pipe, USB_U32 Size, USB_U8 *Table)
*----------------------------------------------------------------------------- 
* Function   : 
*            : 
*----------------------------------------------------------------------------- 
* Argument   : USB_U16 Pipe         ; Pipe Number
*            : USB_U32 Size         ; Data Size
*            : USB_U8  *Table       ; Data Table Address
*----------------------------------------------------------------------------- 
* ReturnValue : USB_SUCCESS     : success
*             : USB_ERROR       : error
*----------------------------------------------------------------------------- 
* Notice     : 
*            : 
*""FUNC COMMENT END""*********************************************************/
USB_S16 H_DataIn(USB_U16 Pipe, USB_U32 Size, USB_U8 *Table)
{
	USB_S16		ret;

	if(Pipe == PIPE0){
		return USB_ERROR;
	}

	USB.PIPESEL.BIT.PIPESEL =  Pipe;
	if( USB.PIPECFG.BIT.DIR  == 1 ) {
		return USB_ERROR;
	} 

	if( PipeFlag[Pipe] == PIPE_IDLE ){
		usb_start_recieve_transfer(Pipe, Size, Table);
	}
	else{
		return USB_ERROR;				/* Now pipe is busy	*/
	}

	/* waiting for completing routine			*/
	do{
		if( ( (PipeFlag[Pipe] != PIPE_IDLE) && (PipeFlag[Pipe] != PIPE_WAIT) ) ||
			 ( UsbDtchFlg == USB_YES ) ) {
			break;
		}
	} while(1);

	switch( PipeFlag[Pipe] ) {
	case PIPE_DONE:
		ret = USB_SUCCESS;
		break;

	case PIPE_STALL:
		ret = USB_ERROR;
		break;

	case PIPE_NORES:
		ret = USB_ERROR;
		break;

	default:
		ret = USB_ERROR;
		break;
	}

	usb_stop_transfer(Pipe);

	PipeFlag[Pipe] = PIPE_IDLE;

	return ret;
}

/*""FUNC COMMENT""************************************************************ 
* ID         : 
* Outline    : User data receive
*----------------------------------------------------------------------------- 
* Include    : 
*----------------------------------------------------------------------------- 
* Declaration: USB_S16 H_DataOut(USB_U16 Pipe, USB_U32 Size, USB_U8 *Table);
*----------------------------------------------------------------------------- 
* Function   : 
*            : 
*----------------------------------------------------------------------------- 
* Argument   : USB_U16 Pipe         ; Pipe Number
*            : USB_U32 Size         ; Data Size
*            : USB_U8  *Table       ; Data Table Address
*----------------------------------------------------------------------------- 
* ReturnValue : USB_SUCCESS     : success
*             : USB_ERROR       : error
*----------------------------------------------------------------------------- 
* Notice     : 
*            : 
*""FUNC COMMENT END""*********************************************************/
USB_S16 H_DataOut(USB_U16 Pipe, USB_U32 Size, USB_U8 *Table)
{
	USB_S16		ret;

	if(Pipe == PIPE0){
		return USB_ERROR;
	}

	USB.PIPESEL.BIT.PIPESEL =  Pipe;
	if( USB.PIPECFG.BIT.DIR  == 0 ) {
		return USB_ERROR;
	} 

	if( PipeFlag[Pipe] == PIPE_IDLE ){
		usb_start_send_transfer(Pipe, Size, Table);
	}
	else{
		return USB_ERROR;				/* Now pipe is busy	*/
	}

	/* waiting for completing routine			*/
	do{
		if( ( (PipeFlag[Pipe] != PIPE_IDLE) && (PipeFlag[Pipe] != PIPE_WAIT) ) ||
			 ( UsbDtchFlg == USB_YES ) ) {
			break;
		}
	} while(1);

	switch( PipeFlag[Pipe] ) {
	case PIPE_DONE:
		ret = USB_SUCCESS;
		break;

	case PIPE_STALL:
		ret = USB_ERROR;
		break;

	case PIPE_NORES:
		ret = USB_ERROR;
		break;

	default:
		ret = USB_ERROR;
		break;
	}

	usb_stop_transfer(Pipe);

	PipeFlag[Pipe] = PIPE_IDLE;

	return ret;
}

/*""FUNC COMMENT""************************************************************ 
* ID         : 
* Outline    : USB driver
*----------------------------------------------------------------------------- 
* Include    : 
*----------------------------------------------------------------------------- 
* Declaration: USB_S16 H_ControlTransfer2(USB_U16 Req, USB_U16 Val,
*            :                            USB_U16 Indx, USB_U16 Len, USB_U8 *Buf);
*----------------------------------------------------------------------------- 
* Function   : Enumeration & Detach check
*            : 
*----------------------------------------------------------------------------- 
* Argument   : USB_U16 Req          ; bmRequestType & bRequest
*            : USB_U16 Val          ; wValue
*            : USB_U16 Indx         ; wIndex
*            : USB_U16 Len          ; wLength
*----------------------------------------------------------------------------- 
* ReturnValue : USB_SUCCESS     : success
*             : USB_ERROR       : error
*----------------------------------------------------------------------------- 
* Notice     : 
*            : 
*""FUNC COMMENT END""*********************************************************/

USB_S16 H_CtrlTransfer(USB_U16 Req, USB_U16 Val, USB_U16 Indx,
													 USB_U16 Len, USB_U8 *Buf)
{
	USB_S16		ret;
	USB_U16		ret2;

	do{
		ret2 = H_CtrlTransStart( Req, Val, Indx, Len, Buf);
		if( ret2 == USB_SUCCESS ){
			if( ( (PipeFlag[PIPE0] != PIPE_IDLE) && (PipeFlag[PIPE0] != PIPE_WAIT) ) ||
				 ( UsbDtchFlg == USB_YES ) ) {
				break;
			}
		}
		else{
			return USB_ERROR;
		}
	} while(1);

	switch( PipeFlag[PIPE0] ){
	case PIPE_NORES:
	case PIPE_STALL:
		ret = USB_ERROR;
		break;

	case PIPE_DONE:
		ret = USB_SUCCESS;
		break;

	default:
		ret = USB_ERROR;
		break;
	}

	PipeFlag[PIPE0] = PIPE_IDLE;

	return ret;
}

/*""FUNC COMMENT""*************************************************************
* ID          : 
* Outline     : Set pipe information
*-----------------------------------------------------------------------------
* Include     : 
*-----------------------------------------------------------------------------
* Declaration : USB_S16	H_SetEndpoint( USB_U8 *configdescriptor, 
*             :                        USB_CFG_PIPETBL *user_table );
*-----------------------------------------------------------------------------
* Function    : 
*             : 
*-----------------------------------------------------------------------------
* Argument    : USB_U8 *configdescriptor    : device configration descriptor
*             : USB_CFG_PIPETBL *user_table : pipe table
*-----------------------------------------------------------------------------
* ReturnValue : USB_SUCCESS     : success
*             : USB_ERROR       : error
*-----------------------------------------------------------------------------
* Note        : 
*""FUNC COMMENT END""********************************************************/
USB_S16	H_SetEndpoint( 	USB_U8 *configdescriptor, USB_CFG_PIPETBL *user_table )
{
	USB_U16				ret;
	USB_U16				pipe;
	USB_U32				end_point;
	USB_U32				offset;
	USB_U32				totalLength;
	USB_CFG_PIPETBL		*pipe_table;

	for( pipe = PIPE0; pipe <= MAX_PIPE_NO; ++pipe ){
		UsbSqbit[ pipe ] = 0;
	}

	/*  End Point Search 						*/
	end_point   = 0;
	offset      = configdescriptor[ 0 ];
	totalLength = ( USB_U16 )( configdescriptor[ 2 ] 
								+ ( ( USB_U16 )configdescriptor[ 3 ] << 8 ) );
	do{
		if( configdescriptor[ offset + 1 ] == DT_ENDPOINT ) {
			pipe_table = &user_table[ end_point ];

			if( pipe_table->pipe_number == 0xffff ){
				break;
			}

			ret = H_SetEndpointTable(	pipe_table,
										( USB_U8 *)&configdescriptor[ offset ] );

			if( (ret != DIR_H_IN) && (ret != DIR_H_OUT) ){
				return USB_ERROR;
			}

			++end_point;
		}

		/* Next End Point Search				*/
		offset += configdescriptor[ offset ];

	} while( offset < totalLength );

	return USB_SUCCESS;
}

/*""FUNC COMMENT""*************************************************************
* ID          : 
* Outline     : Clear pipe information
*-----------------------------------------------------------------------------
* Include     : 
*-----------------------------------------------------------------------------
* Declaration : USB_S16	H_ClrEndpoint( USB_CFG_PIPETBL *user_table );
*-----------------------------------------------------------------------------
* Function    : 
*             : 
*-----------------------------------------------------------------------------
* Argument    : USB_CFG_PIPETBL *user_table : pipe table
*-----------------------------------------------------------------------------
* ReturnValue : USB_SUCCESS     : success
*             : USB_ERROR       : error
*-----------------------------------------------------------------------------
* Note        : 
*""FUNC COMMENT END""********************************************************/
USB_S16	H_ClrEndpoint( 	USB_CFG_PIPETBL *user_table )
{
	USB_U16				pipe;

	for( pipe = PIPE0; pipe <= MAX_PIPE_NO; ++pipe ){
		UsbSqbit[ pipe ] = 0;
	}

	while(1){
		if( user_table->pipe_number == 0xffff ){
			break;
		}
		user_table->pipe_cfg 			&= (DBLBFIELD | CNTMDFIELD);
		user_table->pipe_max_pktsize	= 0;
		user_table->pipe_cycle			= 0;

		user_table++;
	}
	return USB_SUCCESS;
}
/*===========================================================================*/
/* End of File */
